/* This file is part of Om.  Copyright (C) 2004 Dave Robillard.
 * 
 * Om is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * Om is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#ifndef AUDIOOUTPUTNODE_H
#define AUDIOOUTPUTNODE_H

#include <string>
#include "OutputNode.h"
#include "List.h"
#include "JackDriver.h"

using std::string;

namespace Om {

class OutputPort;


/** A node to represent output ports on a patch
 *
 * A patch doesn't actually contain any ports, it's port accessor methods
 * directly return ports in this node.  This is why every port added to this
 * module really adds two port objects - one from the internal perspective (ie
 * a port on this node in a patch), and one from the external perspective (ie
 * a port on the patch itself (being a node as well).
 *
 * \ingroup engine
 */
class AudioOutputNode : public OutputNode
{
public:
	AudioOutputNode(const string& path, uint poly, Patch* parent, samplerate srate, size_t buffer_size);
	virtual ~AudioOutputNode();

	void run(size_t nframes) { InternalNode::run(nframes); };
	
	void activate();
	void deactivate();
	void add_to_patch(Patch* p);
	void remove_from_patch(Patch* p);
	void send_creation_messages(lo_address addr) const;
	void send_deletion_messages(lo_address addr) const;

	OutputPort* external_port() const { return m_external_port; }

private:
	ListNode<JackPort>* m_jack_port;
};


} // namespace Om

#endif // AUDIOOUTPUTNODE_H
